# -*- coding: utf8 -*-

import xml.etree.ElementTree as ET
import sys

from docutils import nodes
from docutils.parsers.rst import directives, states
from docutils.nodes import fully_normalize_name, whitespace_normalize_name
from docutils.parsers.rst.roles import set_classes
from docutils.core import publish_string
import codecs

#-- 
#
# Adaptacion de la directiva "image" de docutils
#

alinea_values = ('izquierda', 'centro', 'derecha')

def alinea(argument):
    return directives.choice(argument, alinea_values)

def imagen(name, arguments, options, content, lineno,
          content_offset, block_text, state, state_machine):

    alinea = options.get("alinea")
    if alinea:
        if alinea not in alinea_values:
            error = state_machine.reporter.error(
                'Error en la directiva "imagen": "%s" no es un valor válido '
                'para la opción "alinea". Los valores válidos son: %s' % (alinea, alinea_values),
                nodes.literal_block(block_text, block_text), line=lineno)
            return [error]

    posic = options.get("posic")
    if posic:
        if len(posic) != 2:
            error = state_machine.reporter.error(
                'Error en la directiva "imagen": "%s" no es un valor válido '
                'para la opción "posic". Se necesitan dos enteros.' % posic,
                nodes.literal_block(block_text, block_text), line=lineno)
            return [error]
    if alinea and posic:
            error = state_machine.reporter.error(
                'Error en la directiva "imagen": Se puede utilizar "alinea" o "posic", no ambos.',
                nodes.literal_block(block_text, block_text), line=lineno)
            return [error]
        
    messages = []
    reference = directives.uri(arguments[0])
    options['uri'] = reference
    reference_node = None
    if options.has_key('target'):
        block = states.escape2null(options['target']).splitlines()
        block = [line for line in block]
        target_type, data = state.parse_target(block, block_text, lineno)
        if target_type == 'refuri':
            reference_node = nodes.reference(refuri=data)
        elif target_type == 'refname':
            reference_node = nodes.reference(
                refname=fully_normalize_name(data),
                name=whitespace_normalize_name(data))
            reference_node.indirect_reference_name = data
            state.document.note_refname(reference_node)
        else:                           # malformed target
            messages.append(data)       # data is a system message
        del options['target']
    set_classes(options)
    image_node = nodes.image(block_text, **options)
    if reference_node:
        reference_node += image_node
        return messages + [reference_node]
    else:
        return messages + [image_node]

imagen.arguments = (1, 0, 1)
imagen.options = {
#                 'alt': directives.unchanged,
#                 'width': directives.length_or_percentage_or_unitless,
#                 'scale': directives.nonnegative_int,
                 'alinea': alinea,
#                 'target': directives.unchanged_required,
#                 'class': directives.class_option,
                 'posic': directives.positive_int_list,
                 }

directives.register_directive("imagen", imagen)
#--

class Section(object):
    def __init__(self, section):
        self.titulo = ""
        self.parrafos = []
        for nodo in section:
            if nodo.tag == "title":
                self.titulo = nodo.text
            else:
                try:
                    tipo = selector[nodo.tag]
                except KeyError:
                    print "Nodo en Section no parseable", nodo.tag
#                    import pdb; pdb.set_trace()
                for res in tipo(nodo):
                    self.parrafos.append(res)

class ElemBase(object):
    tipo = None
    def __init__(self, nodo):
        self.pausa = False
        for k,v in nodo.items():
            if k == "classes" and "pausa" in v.split():
                self.pausa = True

def Problemas(nodo):
    print "Hubo problemas al traducir el archivo ReST!"
    sys.exit()

def Block(nodo):
    "No nos importan los bloques, solo entramos al primer nodo"
    # warning: ¿solo tienen un nodo?
    nodo = nodo[0]
#    print "Block!", nodo, nodo.tag
    tipo = selector[nodo.tag]
    res = tipo(nodo)
    return res

def gen_imagen(nodo):
    i = Imagen(nodo)
    yield i

class Imagen(ElemBase):
    tipo = "imagen"
    def __init__(self, nodo):
        self.uri = nodo.attrib.pop("uri")
        self.alinea = nodo.attrib.pop("alinea", None)
        self.posic = nodo.attrib.pop("posic", None)
        if self.posic:
            self.posic = tuple(int(x) for x in self.posic.split())
        ElemBase.__init__(self, nodo)

def gen_parrafo(nodo):
    p = Parrafo(nodo)
    yield p

class Parrafo(ElemBase):
    tipo = "parrafo"
    def __init__(self, nodo):
        self.texto = nodo.text.replace("\n", " ")
        self.center = False
        for k,v in nodo.items():
            if k == "classes" and "center" in v.split():
                self.center = True
        ElemBase.__init__(self, nodo)

#        import pdb; pdb.set_trace()
def gen_bullets(nodo):
    caractbull = nodo.attrib.pop("bullet")
    for elem in nodo:
        assert elem.tag == "list_item"
        bull = Bullets(caractbull, elem)
        yield bull

class Bullets(ElemBase):
    tipo = "bullets"
    def __init__(self, caractbull, nodo):
        self.caractbull = caractbull
        self.elementos = []
        for elem in nodo:
#            import pdb; pdb.set_trace()
            try:
                tipo = selector[elem.tag]
            except KeyError:
                print "Nodo en Section no parseable", elem.tag
            for res in tipo(elem):
                self.elementos.append(res)
        ElemBase.__init__(self, nodo)

selector = {
    "paragraph":        gen_parrafo,
    "bullet_list":      gen_bullets,
    "image":            gen_imagen,
    "system_message":   Problemas,
    "block_quote":      Block,
    "list_item":        Bullets,
}


class RestXML(object):
    def __init__(self, nomarch):
        import locale
        locale.setlocale(locale.LC_ALL, '')

        textorest = codecs.open(nomarch, encoding="utf8").read()
        exportedxml = publish_string(source=textorest, source_path=textorest, writer_name='xml', settings_overrides={'input_encoding': 'utf8'})

        self.titulo = self.autores = ""
        self.secciones = []
        xml = ET.fromstring(exportedxml)
        self._getElements(xml)

    def _getElements(self, xml):
        for nodo in xml:
            nomfunc = "parse_" + nodo.tag
            if hasattr(self, nomfunc):
                func = getattr(self, nomfunc)
                func(nodo)
            else:
                print "Nodo no parseable: '%s'" % nodo.tag


    # --------------------------
    # automaticos

    def parse_title(self, nodo):
        self.titulo = nodo.text

    def parse_docinfo(self, nodo):
        for nod in nodo:
            if nod.tag == "author":
                self.autores = nod.text

    def parse_section(self, nodo):
        sec = Section(nodo)
#        import pdb; pdb.set_trace()
        self.secciones.append(sec)

